/*********************************************************************************
 * Proiectarea cu Microprocesoare
 * Proiect
 * Persistance of Vision Clock
 * Cristea Adrian - 333CC
 *********************************************************************************/

#include <avr\io.h>
#include <avr\interrupt.h>
#include <util/delay.h>
#include <stdlib.h>

#define F_CPU 16000000

#define SECUNDA_A 0xC0
#define SECUNDA_C 0xFF

#define MINUT 0xCF
#define MINUTS 0x7F

#define ORA 0x3F
#define ORAS 0x1F


//cadranul ceasului va fi impartit in 120 de pozitii

//un vector in care am pastrat ledurice ce trebuiesc aprinse in fiecare pozitie

unsigned char leduri[120] = {0x01, 0x3A, 0x2B, 0x2E, 0x01, 0x00, 0x01, 0x00, 0x01, 0x08,
							0x05, 0x3E, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x3A,
							0x2B, 0x2E, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x22,
							0x2B, 0x3E, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x18,
							0x15, 0x3A, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x2E,
							0x2B, 0x3A, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x3C,
							0x2B, 0x3A, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x26,
							0x13, 0x0E, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x3E,
							0x2B, 0x3E, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x2E,
							0x2B, 0x1E, 0x01, 0x00, 0x01, 0x00, 0x01, 0x08, 0x05, 0x3E,
							0x01, 0x3E, 0x23, 0x3E, 0x01, 0x00, 0x01, 0x08, 0x05, 0x3E,
							0x01, 0x08, 0x05, 0x3E, 0x01, 0x00, 0x01, 0x08, 0x05, 0x3E };


/*unsigned char leduri[120] = {0x01, 0x3B, 0x2B, 0x2F, 0x01, 0x01, 0x01, 0x01, 0x01, 0x09,
							0x05, 0x3F, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x3B,
							0x2B, 0x2F, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x23,
							0x2B, 0x3F, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x19,
							0x15, 0x3B, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x2F,
							0x2B, 0x3B, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x3D,
							0x2B, 0x3B, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x27,
							0x13, 0x0F, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x3F,
							0x2B, 0x3F, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x2F,
							0x2B, 0x1F, 0x01, 0x01, 0x01, 0x01, 0x01, 0x09, 0x05, 0x3F,
							0x01, 0x3F, 0x23, 0x3F, 0x01, 0x01, 0x01, 0x09, 0x05, 0x3F,
							0x01, 0x09, 0x05, 0x3F, 0x01, 0x01, 0x01, 0x09, 0x05, 0x3F };*/

//variabile ce vor retine secunda, minutul si ora							
unsigned char minut, ora, secunda, stai;


/* Initializarea timerului pe 16 biti*/

void initTimer() {
	// mode: CTC prescaler: 1024
	TCCR1B = _BV(WGM12) | _BV(CS12) | _BV(CS10);
	
	// configurarea timer-ul pentru 1 secunda
	OCR1AH = 61;
	OCR1AL = 9;
	
	//	activarea intreruperii
	TIMSK |= _BV(OCIE1A);
	
}


ISR(TIMER1_COMPA_vect) {
	
	// se incrementeaza secunda si se modifica daca este necesar minutul si ora	
	secunda ++;
	if(secunda == 60) {
		secunda = 0;
		minut ++;
		
		if (minut == 60) {
			minut = 0;
			ora ++;
			
			if(ora == 12) {
				ora = 0;
			}
		}
	}
	
	//se decrementeaza numarul de secunde pana se poate seta iar ceasul
	if(stai>0) {
		stai--;
	}
}

int main() {

	int i, j, m, h;

	stai = secunda = ora = 0;
	minut = 30;
	
	TIMSK = 0;
	
	sei();
	
	//se seteaza registrii pinilor microcontrolerului pentru intrare sau iesire
	DDRC = 0xFF;
	DDRA = 0xFF;
	DDRD = 0x00;

	//se initializeaza timerul
	initTimer();
	
	while(1){
		
		//se verifica pozitia limbilor pentru a sti unde vor fi afisate
		if(minut == 0)
			m = 118;
		else
			m = 2 * minut - 2;
			
		if(ora == 0)
			h = 117;
		else
			h = 10 * ora - 3;
			
		for(i=0;i<120;i++) {
		
			//se afiseaza cifrele
			PORTA = leduri[119-i];
			
			//daca linia contine secunda se afiseaza si aceasta
			if(119 - i == 2 * secunda) {
				PORTC = SECUNDA_C;
				PORTA |= SECUNDA_A;
			}
			
			//daca linia contine partea stanga a limbii minutului se afiseaza si aceasta
			else if(119-i == m)
				PORTC = MINUTS;
				
			//daca linia contine centrul limbii minutului se afiseaza si acesta
			else if(119-i == 2*minut)
				PORTC = MINUT;
				
			//daca linia contine partea dreapta a limbii minutului se afiseaza si aceasta
			else if(119-i == (2*minut + 2)%120)
				PORTC = MINUTS;
				
			//daca linia contine partea stanga a limbii orei se afiseaza si aceasta
			else if( 119 - i == h)
				PORTC = ORAS;
				
			//daca linia contine centrul limbii orei se afiseaza si acesta	
			else if( 119 - i == 10 * ora)
				PORTC = ORA;
				
			//daca linia contine partea dreapta a limbii orei se afiseaza si aceasta
			else if( 119 - i == (10 * ora+3)%120)
				PORTC = ORAS;
				
			//se afiseaza centrul ceasului
			else 
				PORTC = 0x01;
			
			//daca s-a luminat sub 3 si e posibila setarea ceasului
			if ((i == 90) && ((PIND & 0x20) == 0x20) && (stai == 0)) {
			
				//se incrementeaza minutul
				minut  = (minut + 1)%60;
				
				//se seteaza numarul de secunde pana cand se poate seta din nou
				stai = 2;
			}
			
			//daca s-a luminat sub 9 si e posibila setarea ceasului
			if ((i == 30) && ((PIND & 0x20) == 0x20) && (stai == 0)) {
			
				//se decremenetaza minutul
				if(minut == 0) 
					minut = 59;
				else 
					minut --;
					
				//se seteaza numarul de secunde pana cand se poate seta din nou
				stai = 2;
			}
			
			//daca s-a luminat sub 6 si e posibila setarea ceasului
			if ((i == 60) && ((PIND & 0x20) == 0x20) && (stai == 0)) {
			
				//se decrementeaza ora
				if(ora == 0) 
					ora = 11;
				else 
					ora --;
					
				//se seteaza numarul de secunde pana cand se poate seta din nou
				stai = 2;
			}
			
			//daca s-a luminat sub 12 si e posibila setarea ceasului
			if ((i == 0) && ((PIND & 0x20) == 0x20) && (stai == 0)) {
			
				//se incrementeaza ora
				ora = (ora + 1)%12;
				stai = 2;
			}
			
			//se asteapta pana se afiseaza urmatoarea linie
			for(j=0;j<600;j++);
		}
	
	}
	
	return 0;
}
